/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>
#include <QPluginLoader>
#include <QFontDatabase>
#include "mainwindow.h"
#ifdef Q_WS_QWS
#include <SDL/SDL.h>
#include <PDL.h>
#include <QWSEvent>
#endif


class QCustomApplication : public QApplication
{
public:
    QCustomApplication ( int & argc, char ** argv );
    QCustomApplication ( int & argc, char ** argv, bool GUIenabled );
    //QCustomApplication ( int & argc, char ** argv, Type type );
    //QCustomApplication ( Display * display, Qt::HANDLE visual = 0, Qt::HANDLE colormap = 0 );
    //QCustomApplication ( Display * display, int & argc, char ** argv, Qt::HANDLE visual = 0, Qt::HANDLE colormap = 0 );

    virtual	~QCustomApplication () {};
    bool qwsEventFilter(QWSEvent *event);
};
QCustomApplication::QCustomApplication ( int & argc, char ** argv )
    : QApplication(argc,argv)
{
}

QCustomApplication::QCustomApplication ( int & argc, char ** argv, bool GUIenabled )
    : QApplication(argc,argv,GUIenabled)
{

}
bool QCustomApplication::qwsEventFilter(QWSEvent *event)
{
    printf("Got a QWS event: %d\n",event->type);
    /*if (event->type == QWSEvent::Mouse) {
        QWSMouseEvent::SimpleData &mouse = event->asMouse()->simpleData;
        isMove = mouse_x_root != mouse.x_root || mouse_y_root != mouse.y_root;
        oldstate = mouse_state;
        mouse_x_root = mouse.x_root;
        mouse_y_root = mouse.y_root;
        mouse_state = mouse.state;*/

    return false;
}

int main(int argc, char * argv[])
{
    int rv;

#ifdef Q_WS_QWS
#warning "Hacks for QWS"
    // ------------------------------------------------------------------------
    // FONT SETUP    FONT SETUP    FONT SETUP    FONT SETUP    FONT SETUP
    // Get the path to the exe...
    // Ideally we should use QApplication::applicationDirPath, but instanciating that
    // one attempts to load the fonts... which aren't defined yet
    char pathexe[1024];
    rv = readlink("/proc/self/exe", pathexe, sizeof(pathexe));
    if(!(rv>0 && rv<sizeof(pathexe)))
    {
        printf("Couldn't get the exe path - terminating\n");
        return -1;
    }
    pathexe[rv]=0;
    printf("path to exe: %s\n",pathexe);
    *strrchr(pathexe,'/')=0;    // Eliminate the application name
    QString fontpath = QString(pathexe)+"/fonts";
    printf("font path: %s\n",fontpath.toStdString().c_str());
    //if(!qputenv("QT_QWS_FONTDIR",fontpath.toAscii()))
    if(!qputenv("QT_QWS_FONTDIR","/usr/share/fonts"))
    {
        printf("Can't set font path");
    }
    //
    // FONT SETUP    FONT SETUP    FONT SETUP    FONT SETUP    FONT SETUP
    // ------------------------------------------------------------------------

    // Hack setting the the display driver
    int argc2 = argc+2;
    char **argv2 = new char*[argc2];
    for(int i=0;i<argc;i++)
    {
        argv2[i] = new char[strlen(argv[i])+1];
        strcpy(argv2[i],argv[i]);
    }
    //char *display[]={"-display","PalmPreSDLFb:100"};
    char *display[]={"-display","Transformed:PalmPreSDLFb:1"};
    argv2[argc] = display[0];
    argv2[argc+1] = display[1];
    for(unsigned i=0;i<argc2;i++)
    {
        printf("%d: %s\n",i,argv2[i]);
    }

    // QApplication::GuiServer is akin to passing -qws on the command line
    // Passes argv2 / argc2 which stays alive for the entire duration of QApplication
    QApplication app(argc2, argv2,QApplication::GuiServer);
    //QApplication app(argc, argv);
    //QCustomApplication app(argc, argv);
    //QCustomApplication app(argc2, argv2,QApplication::GuiServer);

#else
#warning "No QWS, all OK"
    QApplication app(argc, argv,QApplication::GuiServer);
#endif


    QUrl url;
    if (argc > 1)
        url = QUrl(argv[1]);
    else
       //url = QUrl("http://www.google.com/ncr");
       url = QUrl("http://www.slashdot.org");

    MainWindow *browser = new MainWindow(url);

    #if defined Q_OS_SYMBIAN || defined Q_WS_HILDON || defined Q_WS_MAEMO_5 || defined Q_WS_SIMULATOR || defined Q_WS_QWS
        browser->showMaximized();
    #else
        browser->show();
    #endif

/*
    const int width = 640;
       const int height = 480;

       QGraphicsScene scene;

       QGraphicsView view(&scene);
       view.setFrameShape(QFrame::NoFrame);
       view.setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
       view.setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

       QGraphicsWebView webview;
       webview.resize(width, height);
       webview.load(QUrl("http://www.slashdot.org"));

       scene.addItem(&webview);

       view.resize(width, height);
       view.show();*/

    rv = app.exec();
    return rv;

}
