/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>
#include <QtWebKit>
#include "mainwindow.h"

//#undef QT_NO_QWS_TRANSFORMED
#include <QTransformedScreen>
#include <QWSEvent>
#include "cio.h"

//! [1]

//#define USEGWEV



bool Filter::eventFilter(QObject *obj, QEvent *event)
{
   printf("Filter: Event %d\n",event->type());
    /*if (event->type() == QEvent::KeyPress) {
        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
        qDebug("Ate key press %d", keyEvent->key());
        return true;
    } else {
        // standard event processing
        return QObject::eventFilter(obj, event);
    }*/
   return QObject::eventFilter(obj,event);
}

MobileWebView::MobileWebView(QGraphicsItem *parent)
   : QGraphicsWebView(parent)
{

}

MobileWebView::~MobileWebView()
{

}
void MobileWebView::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
   printf("mouse press\n");
   return QGraphicsWebView::mousePressEvent(event);
}

void MobileWebView::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event )
{
   printf("mouse release");
   return QGraphicsWebView::mouseReleaseEvent(event);
}

void MobileWebView::mouseMoveEvent ( QGraphicsSceneMouseEvent * event )
{
   printf("mouse move\n");
   return QGraphicsWebView::mouseMoveEvent(event);
}

/*
QRectF MobileWebView::boundingRect() const
{
   qreal penWidth = 1;
   return QRectF(-10 - penWidth / 2, -10 - penWidth / 2,
   20 + penWidth, 20 + penWidth);
}

void MobileWebView::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,QWidget *widget)
{
   painter->drawRoundedRect(-10, -10, 20, 20, 5, 5);
}*/
/*
bool MobileWebView::mousePress(const QPoint &value)
{
   return QGraphicsWidget::mousePress(value);
}
void MobileWebView::mouseMove(const QPoint &value)
{
   return QGraphicsWidget::mouseMove(value);
}

void MobileWebView::mouseRelease(const QPoint &value)
{
   return QGraphicsWidget::mouseRelease(value);
}*/


MainWindow::MainWindow(const QUrl& url)
{
#ifndef Q_WS_QWS
   ConsoleInit();
#endif
   progress = 0;



   QFile file;
   file.setFileName(":/jquery.min.js");
   file.open(QIODevice::ReadOnly);
   jQuery = file.readAll();
   file.close();
   printf("jquery: %d\n",jQuery.size());
   //! [1]

   QNetworkProxyFactory::setUseSystemConfiguration(true);

   //! [2]
#ifdef USEGWEV
   //gwebview  = new QGraphicsWebView;
   gwebview  = new MobileWebView;
   gwebview->resize(800,480);
   gwebview->load(url);
   //mobileview = new MobileWebView();

   Filter *filter = new Filter();
   gwebview->installEventFilter(filter);
#else
   webview = new QWebView(this);
   //webview = new MobileWebView(this);
   printf("After new webview...\n");
   webview->load(url);
#endif



   printf("After load url...\n");
#ifdef USEGWEV
    connect(gwebview, SIGNAL(loadFinished(bool)), SLOT(adjustLocation()));
    connect(gwebview, SIGNAL(titleChanged(QString)), SLOT(adjustTitle()));
    connect(gwebview, SIGNAL(loadProgress(int)), SLOT(setProgress(int)));
    connect(gwebview, SIGNAL(loadFinished(bool)), SLOT(finishLoading(bool)));
#else
    connect(webview, SIGNAL(loadFinished(bool)), SLOT(adjustLocation()));
    connect(webview, SIGNAL(titleChanged(QString)), SLOT(adjustTitle()));
    connect(webview, SIGNAL(loadProgress(int)), SLOT(setProgress(int)));
    connect(webview, SIGNAL(loadFinished(bool)), SLOT(finishLoading(bool)));
#endif
   printf("After connection...\n");

   locationEdit = new QLineEdit(this);
   locationEdit->setSizePolicy(QSizePolicy::Expanding, locationEdit->sizePolicy().verticalPolicy());
   connect(locationEdit, SIGNAL(returnPressed()), SLOT(changeLocation()));

   QToolBar *toolBar = addToolBar(tr("Navigation"));

#ifdef USEGWEV
    toolBar->addAction(gwebview->pageAction(QWebPage::Back));
    toolBar->addAction(gwebview->pageAction(QWebPage::Forward));
    toolBar->addAction(gwebview->pageAction(QWebPage::Reload));
    toolBar->addAction(gwebview->pageAction(QWebPage::Stop));
#else
    toolBar->addAction(webview->pageAction(QWebPage::Back));
    toolBar->addAction(webview->pageAction(QWebPage::Forward));
    toolBar->addAction(webview->pageAction(QWebPage::Reload));
    toolBar->addAction(webview->pageAction(QWebPage::Stop));
#endif
   toolBar->addWidget(locationEdit);
   //! [2]

   QMenu *viewMenu = menuBar()->addMenu(tr("&View"));
   QAction* viewSourceAction = new QAction("Page Source", this);
   connect(viewSourceAction, SIGNAL(triggered()), SLOT(viewSource()));
   viewMenu->addAction(viewSourceAction);

   //! [3]
   QMenu *effectMenu = menuBar()->addMenu(tr("&Effect"));
   effectMenu->addAction("Highlight all links", this, SLOT(highlightAllLinks()));

   rotateAction = new QAction(this);
   rotateAction->setIcon(style()->standardIcon(QStyle::SP_FileDialogDetailedView));
   rotateAction->setCheckable(true);
   rotateAction->setText(tr("Turn images upside down"));
   connect(rotateAction, SIGNAL(toggled(bool)), this, SLOT(rotateImages(bool)));
   effectMenu->addAction(rotateAction);

   QMenu *toolsMenu = menuBar()->addMenu(tr("&Tools"));
   toolsMenu->addAction(tr("Remove GIF images"), this, SLOT(removeGifImages()));
   toolsMenu->addAction(tr("Remove all inline frames"), this, SLOT(removeInlineFrames()));
   toolsMenu->addAction(tr("Remove all object elements"), this, SLOT(removeObjectElements()));
   toolsMenu->addAction(tr("Remove all embedded elements"), this, SLOT(removeEmbeddedElements()));

   QMenu *aboutMenu = menuBar()->addMenu(tr("&About"));
   aboutMenu->addAction("About Qt",this,SLOT(aboutQt()));

   QMenu *rotateMenu = menuBar()->addMenu(tr("&Rotate"));
   rotateMenu->addAction("Rotate",this,SLOT(rotateWin()));

   setUnifiedTitleAndToolBarOnMac(true);

   printf("...\n");

#ifdef USEGWEV

   printf("New scene\n");
   gscene = new QGraphicsScene;

   //printf("scene->addItem\n");
   gscene->addItem(gwebview);
   //gscene->addItem(mobileview);


   printf("afterscene->addItem\n");

   gview = new QGraphicsView(gscene);



   //const int width = 640;
   //const int height = 480;

   gwebview->resize(480,800);
   //gview->setFrameShape(QFrame::NoFrame);
   gview->setFrameShape(QFrame::Panel);
   //gview->scale(1.6,0.9);
   gwebview->setResizesToContents(true);

   //gview->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   //gview->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

   printf("before set central widget\n");
   setCentralWidget(gview);
   printf("after set central widget\n");

   QsKineticScroller *qs = new QsKineticScroller;
   qs->enableKineticScrollFor(gview);

   QWebSettings::globalSettings()->setAttribute(QWebSettings::TiledBackingStoreEnabled, true);
   //QWebSettings::globalSettings()->setAttribute(QWebSettings::FrameFlatteningEnabled, true);

#else
    setCentralWidget(webview);
    webview->page()->currentFrame()->setScrollBarPolicy(Qt::Horizontal,Qt::ScrollBarAlwaysOn);
    webview->page()->currentFrame()->setScrollBarPolicy(Qt::Vertical,Qt::ScrollBarAlwaysOn);
    qApp->setStyle(new Style());
#endif



    //view.resize(width, height);
    //view.show();

    //qApp->setStyle(new Style());



    file.setFileName(":/style.sty");
    file.open(QIODevice::ReadOnly | QIODevice::Text);
    QByteArray filedata = file.readAll();
    file.close();
    qApp->setStyleSheet(filedata);


    QStyle *s = style();



    geom();
    /*QTimer::singleShot(4000, this, SLOT(rotate0()));
    QTimer::singleShot(8000, this, SLOT(rotate90()));
    QTimer::singleShot(12000, this, SLOT(rotate180()));
    QTimer::singleShot(16000, this, SLOT(rotate270()));
    QTimer::singleShot(20000, this, SLOT(rotate0()));*/
    //static_cast<QTransformedScreen*>(QTransformedScreen::instance())->setTransformation(QTransformedScreen::None);


    printf("MainWindow: this is: %p\n",this);
    printf("Desktop: %p\n",QApplication::desktop());
}
//! [3]

void MainWindow::viewSource()
{
#ifdef USEGWEV
   QNetworkAccessManager* accessManager = gwebview->page()->networkAccessManager();
   QNetworkRequest request(gwebview->url());
#else
    QNetworkAccessManager* accessManager = webview->page()->networkAccessManager();
    QNetworkRequest request(webview->url());
#endif
    QNetworkReply* reply = accessManager->get(request);
    connect(reply, SIGNAL(finished()), this, SLOT(slotSourceDownloaded()));
}

void MainWindow::slotSourceDownloaded()
{
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(const_cast<QObject*>(sender()));
    QTextEdit* textEdit = new QTextEdit(NULL);
    textEdit->setAttribute(Qt::WA_DeleteOnClose);
    textEdit->show();
    textEdit->setPlainText(reply->readAll());
    reply->deleteLater();
}

//! [4]
void MainWindow::adjustLocation()
{
#ifdef USEGWEV
   locationEdit->setText(gwebview->url().toString());
#else
   locationEdit->setText(webview->url().toString());
#endif


}

void MainWindow::changeLocation()
{
    QUrl url = QUrl(locationEdit->text());
#ifdef USEGWEV
    gwebview->load(url);
    gwebview->setFocus();
#else
    webview->load(url);
    webview->setFocus();
#endif

}
//! [4]

//! [5]
void MainWindow::adjustTitle()
{
#ifdef USEGWEV
   if (progress <= 0 || progress >= 100)
       setWindowTitle(gwebview->title());
   else
       setWindowTitle(QString("%1 (%2%)").arg(gwebview->title()).arg(progress));
#else
   if (progress <= 0 || progress >= 100)
       setWindowTitle(webview->title());
   else
       setWindowTitle(QString("%1 (%2%)").arg(webview->title()).arg(progress));
#endif

}

void MainWindow::setProgress(int p)
{
    progress = p;
    adjustTitle();
}
//! [5]

//! [6]
void MainWindow::finishLoading(bool)
{
   progress = 100;
   adjustTitle();

#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(jQuery);
#else
   webview->page()->mainFrame()->evaluateJavaScript(jQuery);
#endif



    rotateImages(rotateAction->isChecked());
}
//! [6]

//! [7]
void MainWindow::highlightAllLinks()
{
   QString code = "$('a').each( function () { $(this).css('background-color', 'yellow') } )";
#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(code);
#else
   webview->page()->mainFrame()->evaluateJavaScript(code);
#endif


}
//! [7]

//! [8]
void MainWindow::rotateImages(bool invert)
{
   QString code;
   if (invert)
       code = "$('img').each( function () { $(this).css('-webkit-transition', '-webkit-transform 2s'); $(this).css('-webkit-transform', 'rotate(180deg)') } )";
   else
       code = "$('img').each( function () { $(this).css('-webkit-transition', '-webkit-transform 2s'); $(this).css('-webkit-transform', 'rotate(0deg)') } )";
#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(code);
#else
   webview->page()->mainFrame()->evaluateJavaScript(code);
#endif


}
//! [8]

//! [9]
void MainWindow::removeGifImages()
{
   QString code = "$('[src*=gif]').remove()";
#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(code);
#else
   webview->page()->mainFrame()->evaluateJavaScript(code);
#endif




}

void MainWindow::removeInlineFrames()
{
   QString code = "$('iframe').remove()";
#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(code);
#else
   webview->page()->mainFrame()->evaluateJavaScript(code);
#endif


}

void MainWindow::removeObjectElements()
{
   QString code = "$('object').remove()";
#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(code);
#else
   webview->page()->mainFrame()->evaluateJavaScript(code);
#endif


}

void MainWindow::removeEmbeddedElements()
{
   QString code = "$('embed').remove()";
#ifdef USEGWEV
   gwebview->page()->mainFrame()->evaluateJavaScript(code);
#else
   webview->page()->mainFrame()->evaluateJavaScript(code);
#endif


}
//! [9]

void MainWindow::aboutQt()
{
    printf("msgbox\n");
    QMessageBox::aboutQt(this,"About qt - let's check it out!");
}

void MainWindow::rotateWin()
{
    rotate90();



}


void MainWindow::rotate0()
{
#ifdef Q_WS_QWS
    printf("rotate0\n");
    //QTransformedScreen *ts = static_cast<QTransformedScreen*>(QTransformedScreen::instance());
    QTransformedScreen *ts = static_cast<QTransformedScreen*>(QScreen::instance());
    printf("ts: %p\n",ts);
    ts->setTransformation(QTransformedScreen::None);
    /*ts->setDirty(QRect(0,0,800,800));
    showNormal();
    showMaximized();
    locationEdit->setFocus(Qt::MouseFocusReason);
    this->repaint();
    update();*/
    geom();
    printf("done\n");
#endif
}

void MainWindow::rotate90()
{
#ifdef Q_WS_QWS
    printf("rotate90\n");
    QTransformedScreen *ts = static_cast<QTransformedScreen*>(QTransformedScreen::instance());
    printf("ts: %p\n",ts);
    ts->setTransformation(QTransformedScreen::Rot90);
    /*ts->setDirty(QRect(0,0,800,800));
    showNormal();
    showMaximized();
    locationEdit->setFocus(Qt::MouseFocusReason);
    this->repaint();
    update();*/
    geom();
    printf("done\n");
#endif
}
void MainWindow::rotate180()
{
#ifdef Q_WS_QWS
    printf("rotate180\n");
    QTransformedScreen *ts = static_cast<QTransformedScreen*>(QTransformedScreen::instance());
    printf("ts: %p\n",ts);
    ts->setTransformation(QTransformedScreen::Rot180);
    /*ts->setDirty(QRect(0,0,800,800));
    showNormal();
    showMaximized();
    locationEdit->setFocus(Qt::MouseFocusReason);
    this->repaint();
    update();*/
    geom();
    printf("done\n");
#endif
}
void MainWindow::rotate270()
{
#ifdef Q_WS_QWS
    printf("rotate270\n");
    QTransformedScreen *ts = static_cast<QTransformedScreen*>(QTransformedScreen::instance());
    printf("ts: %p\n",ts);
    ts->setTransformation(QTransformedScreen::Rot270);
    /*ts->setDirty(QRect(0,0,800,800));
    showNormal();
    showMaximized();
    locationEdit->setFocus(Qt::MouseFocusReason);
    this->repaint();
    update();*/
    geom();
    printf("done\n");
#endif
}
void MainWindow::geom()
{
    QDesktopWidget* desktopWidget = QApplication::desktop();
    //get client rect.
    QRect clientRect = desktopWidget->availableGeometry();
    printf("clientRect: %d %d %d %d\n",clientRect.top(),clientRect.left(),clientRect.width(),clientRect.height());

    //get application rect.
    QRect applicationRect = desktopWidget->screenGeometry();
    printf("applicationRect: %d %d %d %d\n",applicationRect.top(),applicationRect.left(),applicationRect.width(),applicationRect.height());
}



bool MainWindow::qwsEvent ( QWSEvent * event )
{
    printf("MainWindows::qwsEvent %d\n",event->type);
    return false;
}
void MainWindow::resizeEvent(QResizeEvent *event)
{
   printf("MainWindows: resized to: %d %d\n",event->size().width(),event->size().height());
#ifdef USEGWEV
//   gwebview->resize(gview->size());
#endif
}
